/*
 * Copyright (c) 2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <device.h>
#include <devicetree.h>
#include <drivers/gpio.h>
#include <drivers/i2c.h>

/* 1000 msec = 1 sec */
#define SLEEP_TIME_MS   100

/* The devicetree node identifier for the "led0" alias. */
#define LED0_NODE DT_ALIAS(led0)

#if DT_NODE_HAS_STATUS(LED0_NODE, okay)
#define LED0	DT_GPIO_LABEL(LED0_NODE, gpios)
#define PIN	DT_GPIO_PIN(LED0_NODE, gpios)
#define FLAGS	DT_GPIO_FLAGS(LED0_NODE, gpios)
#else
/* A build error here means your board isn't set up to blink an LED. */
#error "Unsupported board: led0 devicetree alias is not defined"
#define LED0	""
#define PIN	0
#define FLAGS	0
#endif

const struct device * i2c_dev;

uint8_t i2c_sensor_read_reg(uint8_t register_address)
{
    int error;
    uint8_t i2c_read_data;

    error = i2c_write_read(i2c_dev, 0x38, &register_address, 1, &i2c_read_data, 1);

    if(error != 0) {
      printk("i2c_write_read() error: %d!\n", error);
      return 0;
    }

    return i2c_read_data;
}

void main(void)
{
	const struct device *dev;
	bool led_is_on = true;
	int ret;

	printk("Blinky with I2C example started\n");

	dev = device_get_binding(LED0);
	if (dev == NULL) {
		return;
	}

	ret = gpio_pin_configure(dev, PIN, GPIO_OUTPUT_ACTIVE | FLAGS);
	if (ret < 0) {
		return;
	}

	// Get the device binding to the I2C_1 peripheral
	i2c_dev = device_get_binding("I2C_1");
	if (i2c_dev == NULL) {
		printk("Unable to bind I2C_1!\n");
		return;
	}
	
	// Scan the I2C bus for any connected slaves, and print out the address if found
	for(uint8_t i2c_addr = 1; i2c_addr <= 127; i2c_addr++){
		uint8_t i2c_cmd = 1;
		int error = i2c_write(i2c_dev, &i2c_cmd, 1, i2c_addr);
		if(error == 0){
			printk("I2C slave found on address %x\n", i2c_addr);
		}
	}

	while (1) {
		// Read the number of touch points from the sensor, and print out x and y coordinates if > 0 
		int touch_point_num = i2c_sensor_read_reg(0x02);
		if(touch_point_num > 0){
			uint8_t x = i2c_sensor_read_reg(0x04);
			uint8_t y = i2c_sensor_read_reg(0x06);
			printk("Touch coordinates: %i, %i\n", x, y);
		}else {
			printk("Fingers on screen: %i\n", touch_point_num);
		}

		gpio_pin_set(dev, PIN, (int)led_is_on);
		led_is_on = !led_is_on;
		k_msleep(SLEEP_TIME_MS);
	}
}
